#! /usr/bin/python3

# requires python3
# pythonping
# allantools

import math
import ping3
import sys
import time

n_samples = int(sys.argv[2])

print('Determine sane interval')

max_ = -1.

for it in range(0, 100):
    took = ping3.ping(sys.argv[1], timeout=1)

    if took != None and took > max_:
        max_ = took

interval = max_ * 5  # a bit headroom

if interval < 0.01:
    interval = 0.01

print(f'Interval: {interval}s, {n_samples} samples')

# unfortunately even on a LAN, pings sometimes get lost
compensate = 0

timestamps = []

start_t = time.time()

it = 0

while len(timestamps) < n_samples:
    # +1 to begin with a busy loop, to warm up
    next_ = start_t + (it + 1) * interval

    while time.time() < next_:  # yes, this is a busy loop
        pass

    took = ping3.ping(sys.argv[1], timeout=1.0)

    if took == None or took > interval * 0.95:
        print(f'unexpected long interval ({took}) at {it} samples')

        compensate += 1

    else:
        timestamps.append(time.time() - compensate * interval)

    it += 1

print('Done measuring')

import allantools

a = allantools.Dataset(data=timestamps, rate=1./interval)
a.compute("adev")

b = allantools.Plot()
b.plot(a, errorbars=True, grid=True, marker='x')
b.ax.set_xlabel("Tau (s)")
b.ax.set_title(f'{sys.argv[1]}, interval {interval:.6f}s')
b.show()
